@@ -15,6 +15,8 @@ use Data::Dumper;
 my $ROTS_B = 500;
 my $ROTS_K = 5000;
 
+my $MIN_REPS_MIN_CPM = "2,1";
+
 
 my $usage = <<__EOUSAGE__;
 
@@ -25,7 +27,7 @@ my $usage = <<__EOUSAGE__;
 #
 #  --matrix|m <string>               matrix of raw read counts (not normalized!)
 #
-#  --method <string>               edgeR|DESeq2|voom|ROTS
+#  --method <string>               edgeR|DESeq2|voom
 #                                     note: you should have biological replicates.
 #                                           edgeR will support having no bio replicates with
 #                                           a fixed dispersion setting. 
@@ -42,7 +44,10 @@ my $usage = <<__EOUSAGE__;
 #
 #  General options:
 #
-#  --min_rowSum_counts <int>       default: 2  (only those rows of matrix meeting requirement will be tested)
+#  --min_reps_min_cpm  <string>    default: $MIN_REPS_MIN_CPM  (format: 'min_reps,min_cpm')
+#                                  At least min count of replicates must have cpm values > min cpm value.
+#                                     (ie. filtMatrix = matrix[rowSums(cpm(matrix)> min_cpm) >= min_reps, ]  adapted from edgeR manual)
+#                                      Note, ** if no --samples_file, default for min_reps is set = 1 **
 #
 #  --output|o                      name of directory to place outputs (default: \$method.\$pid.dir)
 #
@@ -62,10 +67,6 @@ my $usage = <<__EOUSAGE__;
 #
 #  --dispersion <float>            edgeR dispersion value (Read edgeR manual to guide your value choice)
 #                                    http://www.bioconductor.org/packages/release/bioc/html/edgeR.html
-#  ## ROTS parameters
-#  --ROTS_B <int>                   : number of bootstraps and permutation resampling (default: $ROTS_B)
-#  --ROTS_K <int>                   : largest top genes size (default: $ROTS_K)
-#
 #
 ###############################################################################################
 #
@@ -75,7 +76,6 @@ my $usage = <<__EOUSAGE__;
 #  edgeR:       http://www.bioconductor.org/packages/release/bioc/html/edgeR.html
 #  DESeq2:      http://bioconductor.org/packages/release/bioc/html/DESeq2.html    
 #  voom/limma:  http://bioconductor.org/packages/release/bioc/html/limma.html
-#  ROTS:        http://www.btk.fi/research/research-groups/elo/software/rots/
 #
 ###############################################################################################
 
@@ -87,10 +87,17 @@ __EOUSAGE__
     ;
 
 
+#  ## ROTS parameters
+#  --ROTS_B <int>                   : number of bootstraps and permutation resampling (default: $ROTS_B)
+#  --ROTS_K <int>                   : largest top genes size (default: $ROTS_K)
+#
+#  ROTS:        http://www.btk.fi/research/research-groups/elo/software/rots/
+
+
 my $matrix_file;
 my $method;
 my $samples_file;
-my $min_rowSum_counts = 2;
+
 my $help_flag;
 my $output_dir;
 my $dispersion; # I've used 0.1 myself - but read the manual to guide your choice.
@@ -107,9 +114,9 @@ my $make_tar_gz_file = 0;
               'method=s' => \$method,
               'samples_file|s=s' => \$samples_file,
               'output|o=s' => \$output_dir,
-              'min_rowSum_counts=i' => \$min_rowSum_counts,
+              'min_reps_min_cpm=s' => \$MIN_REPS_MIN_CPM,
               'dispersion=f' => \$dispersion,
-    
+              
               'reference_sample=s' => \$reference_sample,
               'contrasts=s' => \$contrasts_file,
               
@@ -139,9 +146,9 @@ unless ($matrix_file
     
 }
 
-if ($matrix_file =~ /fpkm/i) {
+if ($matrix_file =~ /fpkm|tpm/i) {
     die "Error, be sure you're using a matrix file that corresponds to raw counts, and not FPKM values.\n"
-        . "If this is correct, then please rename your file, and remove fpkm from the name.\n\n";
+        . "If this is correct, then please rename your file, and remove fpkm or tpm from the name.\n\n";
 }
 
 
@@ -149,6 +156,17 @@ unless ($method =~ /^(edgeR|DESeq2|voom|ROTS|GLM)$/) {
     die "Error, do not recognize --method [$method]";
 }
 
+my ($MIN_REPS, $MIN_CPM) = split(/,/, $MIN_REPS_MIN_CPM);
+
+if ($samples_file) {
+    unless ($MIN_REPS > 0 && $MIN_CPM > 0) {
+        die "Error, --min_reps_min_cpm $MIN_REPS_MIN_CPM must include values > 0 in comma-delimited format. ex.  '2,1' ";
+    }
+}
+else {
+    print STDERR "-note, no biological replicates identified, so setting min reps = $MIN_REPS.\n";
+    $MIN_REPS = 1;
+}
 
 
 main: {
@@ -290,7 +308,7 @@ sub parse_sample_info {
 
     my %samples;
 
-    open (my $fh, $sample_file) or die $!;
+    open (my $fh, $sample_file) or die "Error, cannot locate samples file: $sample_file";
     while (<$fh>) {
         unless (/\w/) { next; }
         if (/^\#/) { next; } # allow comments
@@ -379,24 +397,28 @@ sub run_edgeR_sample_pair {
 
     ## write R-script to run edgeR
     open (my $ofh, ">$Rscript_name") or die "Error, cannot write to $Rscript_name";
-    
-    print $ofh "library(edgeR)\n";
 
-    print $ofh "\n";
+    print $ofh "if (! require(edgeR)) {\n";
+    print $ofh "   source(\"https://bioconductor.org/biocLite.R\")\n";
+    print $ofh "   biocLite(\"edgeR\")\n";
+    print $ofh "   library(edgeR)\n";
+    print $ofh "}\n\n";
+    
     
     print $ofh "data = read.table(\"$matrix_file\", header=T, row.names=1, com='')\n";
     print $ofh "col_ordering = c(" . join(",", @rep_column_indices) . ")\n";
     print $ofh "rnaseqMatrix = data[,col_ordering]\n";
     print $ofh "rnaseqMatrix = round(rnaseqMatrix)\n";
-    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(rnaseqMatrix)>=$min_rowSum_counts,]\n";
+    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(cpm(rnaseqMatrix) > $MIN_CPM) >= $MIN_REPS,]\n";
     print $ofh "conditions = factor(c(rep(\"$sample_A\", $num_rep_A), rep(\"$sample_B\", $num_rep_B)))\n";
     print $ofh "\n";
     print $ofh "exp_study = DGEList(counts=rnaseqMatrix, group=conditions)\n";
     print $ofh "exp_study = calcNormFactors(exp_study)\n";
     
     if ($num_rep_A > 1 && $num_rep_B > 1) {
-        print $ofh "exp_study = estimateCommonDisp(exp_study)\n";
-        print $ofh "exp_study = estimateTagwiseDisp(exp_study)\n";
+        #print $ofh "exp_study = estimateCommonDisp(exp_study)\n";
+        #print $ofh "exp_study = estimateTagwiseDisp(exp_study)\n";
+        print $ofh "exp_study = estimateDisp(exp_study)\n"; # new recommended way
         print $ofh "et = exactTest(exp_study, pair=c(\"$sample_A\", \"$sample_B\"))\n";
     }
     elsif (!$dispersion) {
@@ -410,21 +432,28 @@ sub run_edgeR_sample_pair {
         print $ofh "et = exactTest(exp_study, pair=c(\"$sample_A\", \"$sample_B\"), dispersion=$dispersion)\n";
     }
     print $ofh "tTags = topTags(et,n=NULL)\n";
-    print $ofh "write.table(tTags, file=\'$output_prefix.edgeR.DE_results\', sep='\t', quote=F, row.names=T)\n";
-
+    print $ofh "result_table = tTags\$table\n";
+    print $ofh "result_table = data.frame(sampleA=\"$sample_A\", sampleB=\"$sample_B\", result_table)\n";
+    
+    ## reset logfc so it's A/B instead of B/A to be consistent with DESeq2
+    print $ofh "result_table\$logFC = -1 * result_table\$logFC\n";
+    
+    print $ofh "write.table(result_table, file=\'$output_prefix.edgeR.DE_results\', sep='\t', quote=F, row.names=T)\n";
+    print $ofh "write.table(rnaseqMatrix, file=\'$output_prefix.edgeR.count_matrix\', sep='\t', quote=F, row.names=T)\n";
+    
     ## generate MA and Volcano plots
     print $ofh "source(\"$FindBin::RealBin/R/rnaseq_plot_funcs.R\")\n";
     print $ofh "pdf(\"$output_prefix.edgeR.DE_results.MA_n_Volcano.pdf\")\n";
-    print $ofh "result_table = tTags\$table\n";
-    print $ofh "plot_MA_and_Volcano(result_table\$logCPM, result_table\$logFC, result_table\$FDR)\n";
+
+    print $ofh "plot_MA_and_Volcano(rownames(result_table), result_table\$logCPM, result_table\$logFC, result_table\$FDR)\n";
     print $ofh "dev.off()\n";
     
     close $ofh;
 
     ## Run R-script
-    my $cmd = "R --vanilla -q < $Rscript_name";
-
-
+    #my $cmd = "R --no-save --no-restore --no-site-file --no-init-file -q < $Rscript_name";
+    my $cmd = "Rscript $Rscript_name";
+    
     eval {
         &process_cmd($cmd);
     };
@@ -466,15 +495,22 @@ sub run_DESeq2_sample_pair {
 
     ## write R-script to run DESeq
     open (my $ofh, ">$Rscript_name") or die "Error, cannot write to $Rscript_name";
-    
-    print $ofh "library(DESeq2)\n";
-    print $ofh "\n";
+    print $ofh "if (! require(edgeR)) {\n";
+    print $ofh "   source(\"https://bioconductor.org/biocLite.R\")\n";
+    print $ofh "   biocLite(\"edgeR\")\n";
+    print $ofh "   library(edgeR)\n";
+    print $ofh "}\n\n";
+    print $ofh "if (! require(DESeq2)) {\n";
+    print $ofh "   source(\"https://bioconductor.org/biocLite.R\")\n";
+    print $ofh "   biocLite(\"DESeq2\")\n";
+    print $ofh "   library(DESeq2)\n";
+    print $ofh "}\n\n";
 
     print $ofh "data = read.table(\"$matrix_file\", header=T, row.names=1, com='')\n";
     print $ofh "col_ordering = c(" . join(",", @rep_column_indices) . ")\n";
     print $ofh "rnaseqMatrix = data[,col_ordering]\n";
     print $ofh "rnaseqMatrix = round(rnaseqMatrix)\n";
-    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(rnaseqMatrix)>=$min_rowSum_counts,]\n";
+    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(cpm(rnaseqMatrix) > $MIN_CPM) >= $MIN_REPS,]\n";
     print $ofh "conditions = data.frame(conditions=factor(c(rep(\"$sample_A\", $num_rep_A), rep(\"$sample_B\", $num_rep_B))))\n";
     print $ofh "rownames(conditions) = colnames(rnaseqMatrix)\n";
     print $ofh "ddsFullCountTable <- DESeqDataSetFromMatrix(\n"
@@ -482,36 +518,42 @@ sub run_DESeq2_sample_pair {
              . "    colData = conditions,\n"
              . "    design = ~ conditions)\n";
     print $ofh "dds = DESeq(ddsFullCountTable)\n";
-    print $ofh "res = results(dds)\n";
 
+    print $ofh "contrast=c(\"conditions\",\"$sample_A\",\"$sample_B\")\n";
+    print $ofh "res = results(dds, contrast)\n";
+    
 
     # adj from: Carsten Kuenne, thx!
     ##recreates baseMeanA and baseMeanB columns that are not created by default DESeq2 anymore
-    print $ofh "baseMeanA <- rowMeans(counts(dds, normalized=TRUE)[,colData(dds)\$condition == \"$sample_A\"])\n";
-    print $ofh "baseMeanB <- rowMeans(counts(dds, normalized=TRUE)[,colData(dds)\$condition == \"$sample_B\"])\n";
+    print $ofh "baseMeanA <- rowMeans(counts(dds, normalized=TRUE)[,colData(dds)\$conditions == \"$sample_A\"])\n";
+    print $ofh "baseMeanB <- rowMeans(counts(dds, normalized=TRUE)[,colData(dds)\$conditions == \"$sample_B\"])\n";
     print $ofh "res = cbind(baseMeanA, baseMeanB, as.data.frame(res))\n";
  
     ##adds an “id” column headline for column 0
-    print $ofh "res = cbind(id=rownames(res), as.data.frame(res))\n";
+    print $ofh "res = cbind(sampleA=\"$sample_A\", sampleB=\"$sample_B\", as.data.frame(res))\n";
 
     print $ofh "res\$padj[is.na(res\$padj)]  <- 1\n"; # Carsten Kuenne
-    
-    ##set row.names to false to accomodate change above
 
+    print $ofh "res = as.data.frame(res[order(res\$pvalue),])\n"; # rank by pvalue
+    
     ## output results
-    print $ofh "write.table(as.data.frame(res[order(res\$pvalue),]), file=\'$output_prefix.DESeq2.DE_results\', sep='\t', quote=FALSE, row.names=F)\n";
+    print $ofh "write.table(res, file=\'$output_prefix.DESeq2.DE_results\', sep='\t', quote=FALSE)\n";
+    print $ofh "write.table(rnaseqMatrix, file=\'$output_prefix.DESeq2.count_matrix\', sep='\t', quote=FALSE)\n";
+    
     
     ## generate MA and Volcano plots
     print $ofh "source(\"$FindBin::RealBin/R/rnaseq_plot_funcs.R\")\n";
     print $ofh "pdf(\"$output_prefix.DESeq2.DE_results.MA_n_Volcano.pdf\")\n";
-    print $ofh "plot_MA_and_Volcano(log2(res\$baseMean+1), res\$log2FoldChange, res\$padj)\n";
+    print $ofh "plot_MA_and_Volcano(rownames(res), log2(res\$baseMean+1), res\$log2FoldChange, res\$padj)\n";
     print $ofh "dev.off()\n";
         
     
     close $ofh;
     
     ## Run R-script
-    my $cmd = "R --vanilla -q < $Rscript_name";
+    #my $cmd = "R --no-save --no-restore --no-site-file --no-init-file -q < $Rscript_name";
+    my $cmd = "Rscript $Rscript_name";
+    
     &process_cmd($cmd);
     
     return;
@@ -555,7 +597,7 @@ sub run_limma_voom_sample_pair {
     print $ofh "col_ordering = c(" . join(",", @rep_column_indices) . ")\n";
     print $ofh "rnaseqMatrix = data[,col_ordering]\n";
     print $ofh "rnaseqMatrix = round(rnaseqMatrix)\n";
-    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(rnaseqMatrix)>=$min_rowSum_counts,]\n";
+    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(cpm(rnaseqMatrix) > $MIN_CPM) >= $MIN_REPS,]\n";
     print $ofh "conditions = factor(c(rep(\"$sample_A\", $num_rep_A), rep(\"$sample_B\", $num_rep_B)))\n";
     print $ofh "\n";
     print $ofh "design = model.matrix(~conditions)\n";
@@ -570,24 +612,26 @@ sub run_limma_voom_sample_pair {
     print $ofh "# output results, including average expression val for each feature\n";
     print $ofh "c = cpm(x)\n";
     print $ofh "m = apply(c, 1, mean)\n";
+    print $ofh "tTags\$logFC = -1 * tTags\$logFC  # make A/B instead of B/A\n";
     print $ofh "tTags2 = cbind(tTags, logCPM=log2(m[rownames(tTags)]))\n";
-    print $ofh "DE_matrix = data.frame(logFC=tTags\$logFC, logCPM=tTags2\$logCPM, PValue=tTags\$'P.Value', FDR=tTags\$'adj.P.Val')\n";
+    print $ofh "DE_matrix = data.frame(sampleA=\"$sample_A\", sampleB=\"$sample_B\", logFC=tTags\$logFC, logCPM=tTags2\$logCPM, PValue=tTags\$'P.Value', FDR=tTags\$'adj.P.Val')\n";
     print $ofh "rownames(DE_matrix) = rownames(tTags)\n";
     print $ofh "write.table(DE_matrix, file=\'$output_prefix.voom.DE_results\', sep='\t', quote=F, row.names=T)\n";
+    print $ofh "write.table(rnaseqMatrix, file=\'$output_prefix.voom.count_matrix\', sep='\t', quote=F, row.names=T)\n";
     
     ## generate MA and Volcano plots
     print $ofh "# MA and volcano plots\n";
     print $ofh "source(\"$FindBin::RealBin/R/rnaseq_plot_funcs.R\")\n";
     print $ofh "pdf(\"$output_prefix.voom.DE_results.MA_n_Volcano.pdf\")\n";
-    print $ofh "plot_MA_and_Volcano(tTags2\$logCPM, tTags\$logFC, tTags\$'adj.P.Val')\n";
+    print $ofh "plot_MA_and_Volcano(rownames(tTags2), tTags2\$logCPM, tTags\$logFC, tTags\$'adj.P.Val')\n";
     print $ofh "dev.off()\n";
     
     close $ofh;
 
     ## Run R-script
-    my $cmd = "R --vanilla -q < $Rscript_name";
-
-
+    #my $cmd = "R --no-save --no-restore --no-site-file --no-init-file -q < $Rscript_name";
+    my $cmd = "Rscript $Rscript_name";
+    
     eval {
         &process_cmd($cmd);
     };
@@ -640,7 +684,7 @@ sub run_ROTS_sample_pair {
     print $ofh "col_ordering = c(" . join(",", @rep_column_indices) . ")\n";
     print $ofh "rnaseqMatrix = data[,col_ordering]\n";
     print $ofh "rnaseqMatrix = round(rnaseqMatrix)\n";
-    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(rnaseqMatrix)>=$min_rowSum_counts,]\n";
+    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(cpm(rnaseqMatrix) > $MIN_CPM) >= $MIN_REPS,]\n";
     print $ofh "conditions = factor(c(rep(\"$sample_A\", $num_rep_A), rep(\"$sample_B\", $num_rep_B)))\n";
     print $ofh "\n";
     print $ofh "design = model.matrix(~conditions)\n";
@@ -652,7 +696,7 @@ sub run_ROTS_sample_pair {
     print $ofh "voom.data = voom(x, design, lib.size=lib_sizes*tmm_norm_factors, plot=F)\n";
     print $ofh "input_voom = voom.data\$E\n";
     print $ofh "# run ROTS for DE analysis\n";
-    print $ofh "res_voom <- ROTS(data=input_voom,groups=conditions,B=$ROTS_B, K=$ROTS_K)\n";
+    print $ofh "res_voom <- ROTS(data=input_voom,groups=as.numeric(conditions),B=$ROTS_B, K=$ROTS_K)\n";
     print $ofh "results = summary(res_voom, fdr=0.1)\n";
 
     print $ofh "# add logFC and logCPM to result table.\n";
@@ -667,24 +711,26 @@ sub run_ROTS_sample_pair {
     print $ofh "logFC = log2(FC)\n";
     print $ofh "results = summary(res_voom, fdr=0.1)\n";
     print $ofh "feature_order = rownames(results)\n";
-    print $ofh "final_table = data.frame(logCPM=log2(m+1)[feature_order], CPM_A=mean_sampleA_cpm[feature_order], CPM_B=mean_sampleB_cpm[feature_order], logFC=logFC[feature_order], results)\n";
-
+    print $ofh "final_table = data.frame(sampleA=\"$sample_A\", sampleB=\"$sample_B\", logCPM=log2(m+1)[feature_order], CPM_A=mean_sampleA_cpm[feature_order], CPM_B=mean_sampleB_cpm[feature_order], logFC=logFC[feature_order], results)\n";
+    
     print $ofh "write.table(final_table, file=\"$output_prefix.ROTS.DE_results\", quote=F, sep='\t')\n";
-        
+    print $ofh "write.table(rnaseqMatrix, file=\"$output_prefix.ROTS.count_matrix\", quote=F, sep='\t')\n";
+    
+    
     ## generate MA and Volcano plots
     print $ofh "# MA and volcano plots\n";
     print $ofh "source(\"$FindBin::RealBin/R/rnaseq_plot_funcs.R\")\n";
     print $ofh "pdf(\"$output_prefix.voom.DE_results.MA_n_Volcano.pdf\")\n";
-    print $ofh "plot_MA_and_Volcano(final_table\$logCPM, final_table\$logFC, final_table\$FDR)\n";
+    print $ofh "plot_MA_and_Volcano(rownames(final_table), final_table\$logCPM, final_table\$logFC, final_table\$FDR)\n";
     print $ofh "dev.off()\n";
     
     
     close $ofh;
 
     ## Run R-script
-    my $cmd = "R --vanilla -q < $Rscript_name";
-
-
+    #my $cmd = "R --no-save --no-restore --no-site-file --no-init-file -q < $Rscript_name";
+    my $cmd = "Rscript $Rscript_name";
+    
     eval {
         &process_cmd($cmd);
     };
@@ -737,7 +783,8 @@ sub run_GLM {
     
     print $ofh "data = read.table(\"$matrix_file\", header=T, row.names=1, com='')\n";
     print $ofh "rnaseqMatrix = round(data)\n";
-    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(rnaseqMatrix)>=$min_rowSum_counts,]\n";
+    print $ofh "rnaseqMatrix = rnaseqMatrix[rowSums(cpm(rnaseqMatrix) > $MIN_CPM) >= $MIN_REPS,]\n";
+
 
     print $ofh "design = model.matrix(~0+groups)\n";
     print $ofh "colnames(design) = levels(groups)\n";
