@@ -204,7 +204,7 @@ void IRKE::populate_Kmers_from_fasta(const string& fasta_filename, bool reassemb
 			
 			string seq = fe.get_sequence();
 			
-			if (seq.length() < kmer_length + 1) {
+			if (seq.length() < kmer_length) {
 				continue;
 			}
 			
@@ -312,6 +312,12 @@ bool IRKE::prune_some_kmers(unsigned int min_count, float min_entropy, bool prun
 }
 
 
+bool IRKE::prune_branched_kmers() {
+
+    return(kcounter.prune_branched_kmers());
+}
+
+
 unsigned long IRKE::get_graph_size() {
 	
 	// proxy call
