@@ -11,33 +11,8 @@ chdir $FindBin::RealBin or die "error, cannot cd to $FindBin::RealBin";
 
 
 
-my @files_to_keep = qw (cleanme.pl 
-                        README
-                        reads.left.fq.gz
-                        reads.right.fq.gz
-                        runMe.sh
-                        __indiv_ex_sample_derived
-                        misc_run_tests
-                        run_abundance_estimation_procedure.sh
 
-                        reads2.left.fq.gz
-                        reads2.right.fq.gz
-
-                        longReads.fa
-                        Makefile
- test_FL.sh
-
-                        __test_runMe_with_jaccard_clip.sh
-                        __test_runMe_with_SE_reads.sh
-
-   runRecTrinity.sh
-                        );
-
-
-my %keep = map { + $_ => 1 } @files_to_keep;
-
-
-`rm -rf ./trinity_out_dir` if (-d "trinity_out_dir");
+`rm -rf ./trinity_out_dir*`;
 `rm -rf ./bowtie_out` if (-d "bowtie_out");
 `rm -rf ./RSEM.stat` if (-d "RSEM.stat");
 `rm -rf ./trinity_single_outdir` if (-d "trinity_single_outdir");
@@ -61,15 +36,13 @@ my %keep = map { + $_ => 1 } @files_to_keep;
 `rm -rf ./kallisto_PE`;
 `rm -rf ./RSEM_PE`;
 `rm -rf ./test_trinity_long_reads`;
-
-
-foreach my $file (<*>) {
-	
-	if (! $keep{$file}) {
-		print STDERR "-removing file: $file\n";
-		unlink($file);
-	}
-}
+`rm -rf ./test_trinity_bowtie2`;
+`rm -rf ./trinity_test_samples_SE`;
+`rm -rf ./trinity_test_samples_PE`;
+`rm -rf ./trinity_piecemeal`;
+`rm -rf ./trinity_test_samples_PE_min_kmer_cov_3`;
+`rm -rf ./trinity_complete`;
+`rm -rf ./trinity_incl_supertrans`;
 
 
 exit(0);
