@@ -8,7 +8,7 @@ use Cwd;
 use Carp;
 
 use Getopt::Long qw(:config no_ignore_case bundling pass_through);
-
+use Data::Dumper;
 
 my %aligner_params = ( 
 
@@ -51,18 +51,22 @@ my %aligner_params = (
     );
 
 my $rsem_add_opts = "";
-my $eXpress_add_opts = "";
+
 my $kallisto_add_opts = "";
 my $salmon_add_opts= "";
-my $salmon_idx_type = 'quasi';
-my $salmon_quasi_kmer_length = 31;
-my $salmon_fmd_kmer_length = 19;
+
+my $salmon_kmer_length = 31;
 
 my $usage = <<__EOUSAGE__;
 
 #########################################################################
 #
+########################
+#  Essential parameters:
+########################
+#
 #  --transcripts <string>           transcript fasta file
+#
 #  --seqType <string>               fq|fa
 # 
 #  If Paired-end:
@@ -74,21 +78,42 @@ my $usage = <<__EOUSAGE__;
 #
 #      --single <string>
 #
+#   or (preferred):
+#
+#      --samples_file <string>    tab-delimited text file indicating biological replicate relationships.
+#                                   ex.
+#                                        cond_A    cond_A_rep1    A_rep1_left.fq    A_rep1_right.fq
+#                                        cond_A    cond_A_rep2    A_rep2_left.fq    A_rep2_right.fq
+#                                        cond_B    cond_B_rep1    B_rep1_left.fq    B_rep1_right.fq
+#                                        cond_B    cond_B_rep2    B_rep2_left.fq    B_rep2_right.fq
+#
+#                      # if single-end instead of paired-end, then leave the 4th column above empty.
+#
+#
+#
 #  --est_method <string>           abundance estimation method.
-#                                        alignment_based:  RSEM|eXpress       
+#                                        alignment_based:  RSEM
 #                                        alignment_free: kallisto|salmon
 #  
-# --output_dir <string>            write all files to output directory
+###################################
+#  Potentially optional parameters:
+###################################
+#
+# --output_dir <string>            write all files to output directory 
+#                                  (note, if using --samples_file, output_dir will be set automatically according to replicate name))
 #  
 #
 #  if alignment_based est_method:
-#       --aln_method <string>            bowtie|bowtie2|(path to bam file) alignment method.  (note: RSEM requires bowtie)
-#                                       (if you already have a bam file, you can use it here instead of rerunning bowtie)
-#
+#       --aln_method <string>            bowtie|bowtie2 alignment method.  (note: RSEM requires either bowtie or bowtie2)
+#                                       
+###########
 # Optional:
-# 
+# #########
+#
 # --SS_lib_type <string>           strand-specific library type:  paired('RF' or 'FR'), single('F' or 'R').
-#                                         (note, no strand-specific mode for kallisto)
+#                                  
+# --samples_idx <int>               restricte processing to sample entry (index starts at one)
+#
 #
 # --thread_count                   number of threads to use (default = 4)
 #
@@ -101,8 +126,6 @@ my $usage = <<__EOUSAGE__;
 #
 #  --prep_reference                 prep reference (builds target index)
 #
-#  --output_prefix <string>         prefix for output files.  Defaults to --est_method setting.
-#
 #
 ########################################
 #
@@ -123,17 +146,12 @@ my $usage = <<__EOUSAGE__;
 #
 #  --bowtie_RSEM <string>          if using 'bowtie', default: \"$aligner_params{bowtie_RSEM}\"
 #  --bowtie2_RSEM <string>         if using 'bowtie2', default: \"$aligner_params{bowtie2_RSEM}\"
+#                                ** if you change the defaults, specify the full set of parameters to use! **
+#
 #  --include_rsem_bam              provide the RSEM enhanced bam file including posterior probabilities of read assignments.
 #  --rsem_add_opts <string>        additional parameters to pass on to rsem-calculate-expression
 #
 ##########################################################################
-#  eXpress opts:
-#
-#  --bowtie_eXpress <string>  default: \"$aligner_params{bowtie_eXpress}\"
-#  --bowtie2_eXpress <string> default: \"$aligner_params{bowtie2_eXpress}\"
-#  --eXpress_add_opts <string>  default: "$eXpress_add_opts"
-#
-##########################################################################
 #  kallisto opts:
 #
 #  --kallisto_add_opts <string>  default: $kallisto_add_opts  
@@ -142,7 +160,6 @@ my $usage = <<__EOUSAGE__;
 #
 #  salmon opts:
 #
-#  --salmon_idx_type <string>    quasi|fmd (defalt: $salmon_idx_type)
 #  --salmon_add_opts <string>    default: $salmon_add_opts
 #
 #
@@ -160,6 +177,10 @@ my $usage = <<__EOUSAGE__;
 #
 #    $0 --transcripts Trinity.fasta --seqType fq --left reads_1.fq --right reads_2.fq --est_method RSEM --aln_method bowtie --trinity_mode --prep_reference --output_dir rsem_outdir
 #
+#   ## Use a samples.txt file:
+#
+#    $0 --transcripts Trinity.fasta --est_method RSEM --aln_method bowtie2 --prep_reference --trinity_mode --samples_file samples.txt --seqType fq  
+#
 #########################################################################
 
 
@@ -174,7 +195,6 @@ my $output_dir;
 my $help_flag;
 my $transcripts;
 my $bam_file;
-my $paired_flag = "";
 my $DEBUG_flag = 0;
 my $SS_lib_type;
 my $thread_count = 4;
@@ -203,6 +223,9 @@ my $trinity_mode;
 my $include_rsem_bam;
 my $coordsort_bam_flag = 0;
 
+my $samples_file = "";
+my $samples_idx = 0;
+
 &GetOptions ( 'help|h' => \$help_flag,
               'transcripts=s' => \$transcripts,
               'name_sorted_bam=s' => \$bam_file,
@@ -219,6 +242,8 @@ my $coordsort_bam_flag = 0;
               'right=s' => \$right,
               'single=s' => \$single,
               'max_ins_size=i' => \$max_ins_size,
+              'samples_file=s' => \$samples_file,
+              'samples_idx=i' => \$samples_idx,
               
               'output_dir=s' => \$output_dir,
       
@@ -228,7 +253,7 @@ my $coordsort_bam_flag = 0;
 
               'include_rsem_bam' => \$include_rsem_bam,
 
-              'output_prefix=s' => \$output_prefix,
+              #'output_prefix=s' => \$output_prefix,
               
               ##  devel opts
               'prep_reference' => \$prep_reference,
@@ -240,19 +265,15 @@ my $coordsort_bam_flag = 0;
               #
               'bowtie_RSEM=s' => \($aligner_params{'bowtie_RSEM'}),
               'bowtie2_RSEM=s' => \($aligner_params{'bowtie2_RSEM'}),
-              'bowtie_eXpress=s' => \($aligner_params{'bowtie_eXpress'}),
-              'bowtie2_eXpress=s' => \($aligner_params{'bowtie2_eXpress'}),
+
               
               'rsem_add_opts=s' => \$rsem_add_opts,
-              'eXpress_add_opts=s' => \$eXpress_add_opts,
               'kallisto_add_opts=s' => \$kallisto_add_opts,
               'salmon_add_opts=s' => \$salmon_add_opts,
     
               'coordsort_bam' => \$coordsort_bam_flag,
-
-             'salmon_idx_type=s' => \$salmon_idx_type,
-             'salmon_quasi_kmer_length=i' => \$salmon_quasi_kmer_length,
-             'salmon_fmd_kmer_length=i' => \$salmon_fmd_kmer_length,
+    
+              'salmon_kmer_length=i' => \$salmon_kmer_length,
     
     );
 
@@ -270,83 +291,84 @@ unless ($est_method) {
     die $usage;
 }
 
-my @EST_METHODS = qw(RSEM express kallisto salmon);
-my %ALIGNMENT_BASED_EST_METHODS = map { + $_ => 1 } qw (RSEM express eXpress);
+my @EST_METHODS = qw(RSEM kallisto salmon);
+my %ALIGNMENT_BASED_EST_METHODS = map { + $_ => 1 } qw (RSEM);
 my %ALIGNMENT_FREE_EST_METHODS = map { + $_ => 1 } qw (kallisto salmon);
 
 
-unless ($output_dir) {
-    die "Error, must specify output directory name via: --output_dir   ";
-}
-
-unless (($est_method && $prep_reference && $transcripts && (! ($single||$left||$right)) ) ## just prep reference
-
-        || ($transcripts && $est_method && $seqType && ($single || ($left && $right))) # do alignment
+unless (
+    
+    ($est_method && $prep_reference && $transcripts && (! ($single||$left||$right||$samples_file)) ) ## just prep reference
+    
+    || 
+    
+    ($transcripts && $est_method && $seqType && ($single || ($left && $right) || $samples_file)) # do alignment
     
     ) {
-
+    
     die "Error, missing parameter. See example usage options below.\n" . $usage;
 }
 
-my $PROCESSING_EXISTING_BAM_FLAG = 0;
 
 if  ($ALIGNMENT_FREE_EST_METHODS{$est_method}) {
     $aln_method = "none";
 }
-else {
-    if ($aln_method =~ /\.bam$/) {
-        if (-s $aln_method) {
-            # no problem, using the bam file
-            $PROCESSING_EXISTING_BAM_FLAG = 1;
-        }
-        else {
-            die "Error, file $aln_method does not exist or is empty.  ";
-        }
-    }
-    elsif ($aln_method !~ /bowtie2?/) {
-        die "Error, --aln_method must be either 'bowtie' or 'bowtie2' ";
-    }
+elsif ($aln_method !~ /bowtie2?/) {
+    die "Error, --aln_method must be either 'bowtie' or 'bowtie2' ";
 }
 
 
-unless ($est_method =~ /^(RSEM|express|kallisto|salmon|none)$/i) {
+unless ($est_method =~ /^(RSEM|kallisto|salmon|none)$/i) {
     die "Error, --est_method @EST_METHODS only\n";
 }
 
 
-$left = &create_full_path($left) if $left;
-$right = &create_full_path($right) if $right;
+my @samples_to_process;
+if ($samples_file) {
+    @samples_to_process = &parse_samples_file($samples_file);
+    if ($samples_idx >= 0) {
+        my $num_samples = scalar(@samples_to_process);
+        if ($samples_idx > $num_samples) {
+            die "Error, sample index $samples_idx > $num_samples num samples ";
+        }
+    }
+}
+elsif ( ($left && $right) || $single) {
+    
+    unless ($output_dir) {
+        die "Error, must specify output directory name via: --output_dir   ";
+    }
+    @samples_to_process = &create_sample_definition($output_dir, $left, $right, $single);
+    
+}
 
-if ($single) {
-    $single = &create_full_path($single);
+
+
+my $PE_mode = 1;
+
+if ($single || (@samples_to_process && $samples_to_process[0]->{single})) {
+    
     unless ($fragment_length) {
         die "Error, specify --fragment_length for single-end reads (note, not the length of the read but the mean fragment length)\n\n";
     }
+
+    $PE_mode = 0;
 }
 
+    
 $transcripts = &create_full_path($transcripts);
 
 $gene_trans_map_file = &create_full_path($gene_trans_map_file) if $gene_trans_map_file;
-
-
-
-if ($left && $left =~ /\.gz$/) {
-    $left = &add_zcat_gz($left);
+if ($gene_trans_map_file && ! -s $gene_trans_map_file) {
+    die "Error, $gene_trans_map_file doesn't exist or is empty";
 }
-if ($right && $right =~ /\.gz$/) {
-    $right = &add_zcat_gz($right);
-}
-if ($single && $single =~ /\.gz$/) {
-    $single = &add_zcat_gz($single);
-}
-
 
 
 if ($SS_lib_type) {
     unless ($SS_lib_type =~ /^(RF|FR|R|F)$/) {
         die "Error, do not recognize SS_lib_type: [$SS_lib_type]\n";
     }
-    if ($left && $right && length($SS_lib_type) != 2 ) {
+    if ($PE_mode && length($SS_lib_type) != 2 ) {
         die "Error, SS_lib_type [$SS_lib_type] is not compatible with paired reads";
     }
 }
@@ -356,7 +378,7 @@ if ( $thread_count !~ /^\d+$/ ) {
 }
 
 
-{  # check for RSEM installation in PATH 
+{  # check for required tools in PATH 
     
     my $missing = 0;
     my @tools = ('samtools');
@@ -370,9 +392,6 @@ if ( $thread_count !~ /^\d+$/ ) {
     if ($est_method =~ /^RSEM$/i) {
         push (@tools, 'rsem-calculate-expression');
     }
-    elsif ($est_method =~ /^express$/i) {
-        push (@tools, 'express');
-    }
     elsif ($est_method eq 'kallisto') {
         push (@tools, 'kallisto');
     }
@@ -389,27 +408,29 @@ if ( $thread_count !~ /^\d+$/ ) {
         }
     }
     if ($missing) {
-        die "Please be sure bowtie and express are installed and the utilities @tools are available via your PATH setting.\n";
+        die "Please be sure the utilities @tools are available via your PATH setting.\n";
     }
 }
 
+
+
 main: {
 
-
-
     if ($trinity_mode && ! $gene_trans_map_file) {
         $gene_trans_map_file = "$transcripts.gene_trans_map";
         my $cmd = "$FindBin::RealBin/support_scripts/get_Trinity_gene_to_trans_map.pl $transcripts > $gene_trans_map_file";
         &process_cmd($cmd) unless (-e $gene_trans_map_file);
     }
+
+
     
     if ($ALIGNMENT_BASED_EST_METHODS{$est_method}) {
         
-        &run_alignment_BASED_estimation();
+        &run_alignment_BASED_estimation(@samples_to_process);
 
     }
     else {
-        &run_alignment_FREE_estimation();
+        &run_alignment_FREE_estimation(@samples_to_process);
     }
 
     exit(0);
@@ -419,12 +440,14 @@ main: {
 
 ####
 sub run_alignment_FREE_estimation {
+    my @samples = @_;
+
     
     if ($est_method eq "kallisto") {
-        &run_kallisto();
+        &run_kallisto(@samples);
     }
     elsif ($est_method eq "salmon") {
-        &run_salmon();
+        &run_salmon(@samples);
     }
     else {
         die "Error, not recognizing est_method: $est_method";
@@ -436,49 +459,48 @@ sub run_alignment_FREE_estimation {
 
 ####
 sub run_alignment_BASED_estimation {
+    my @samples = @_;
+
     
     my $db_index_name = "$transcripts.${aln_method}";
     
 
-    unless ($PROCESSING_EXISTING_BAM_FLAG) {
+    ###############################################
+    ## Prepare transcript database for alignments
+    ###############################################
+    
+    
+    if ($prep_reference) {
         
-        ###############################################
-        ## Prepare transcript database for alignments
-        ###############################################
+        my $cmd = "${aln_method}-build $transcripts $db_index_name";
         
-                    
-        if ($prep_reference) {
-            
-            my $cmd = "${aln_method}-build $transcripts $db_index_name";
-                    
-            unless (-e "$db_index_name.ok") { 
-             
-                if (-e "$db_index_name.started") {
-                    print STDERR "WARNING - looks like the prep for $db_index_name was already started by another process.  Proceeding with caution.\n";
-                }
-                
-                &process_cmd("touch $db_index_name.started");
-                
-                &process_cmd($cmd);
-                
-                rename("$db_index_name.started", "$db_index_name.ok");
+        unless (-e "$db_index_name.ok") { 
             
+            if (-e "$db_index_name.started") {
+                print STDERR "WARNING - looks like the prep for $db_index_name was already started by another process.  Proceeding with caution.\n";
             }
             
-
-        }
-
-        if (! -e "$db_index_name.ok") {
-            die "Error, index $db_index_name not prepared.  Be sure to include parameter '--prep_reference' to first prepare the reference for alignment.";
+            &process_cmd("touch $db_index_name.started");
+            
+            &process_cmd($cmd);
+            
+            rename("$db_index_name.started", "$db_index_name.ok");
+            
         }
         
+        
     }
+
+    if (! -e "$db_index_name.ok") {
+        die "Error, index $db_index_name not prepared.  Be sure to include parameter '--prep_reference' to first prepare the reference for alignment.";
+    }
+        
     
     
     my $rsem_prefix = &create_full_path("$transcripts.RSEM");
     
     if ($est_method eq 'RSEM') {
-
+                
         if ($prep_reference) {
             
             if (-e "$rsem_prefix.rsem.prepped.started") {
@@ -512,84 +534,90 @@ sub run_alignment_BASED_estimation {
     }
     
     
-    unless ( ($left && $right) || $single) {
+    unless (@samples) {
         print STDERR "Only prepping reference. Stopping now.\n";
         exit(0);
     }
+    
+    print STDERR Dumper(\@samples);
+    
+    my $curr_workdir = cwd();
+    foreach my $sample_href (@samples) {
+        chdir $curr_workdir or die "Error, cannot cd to $curr_workdir";
+        # process below will cd into output dir
+        &run_alignment_do_quant($sample_href, $db_index_name, $rsem_prefix);
+    }
 
+}
+
+####
+sub run_alignment_do_quant {
+    my ($sample_href, $db_index_name, $rsem_prefix) = @_;
+
+    my $output_dir = $sample_href->{output_dir};
     
     #####################
     ## Run alignments
     #####################
+    
+    unless (-d $output_dir) {
+        system("mkdir -p $output_dir");
+    }
+    chdir $output_dir or die "Error, cannot cd to output directory $output_dir";
+    
     my $prefix = $output_prefix;
     if ($prefix) {
         $prefix .= "."; # add separator in filename
     }
     my $bam_file = "${prefix}${aln_method}.bam";
     my $bam_file_ok = "$bam_file.ok";
-    if ($PROCESSING_EXISTING_BAM_FLAG) {
-        $bam_file = $aln_method;
-        $bam_file = &create_full_path($bam_file);
-        unless (-e $bam_file_ok) {
-            &process_cmd("touch $bam_file_ok");
-        }
-    }
-    
-    
-    unless (-d $output_dir) {
-        system("mkdir -p $output_dir");
-    }
-    chdir $output_dir or die "Error, cannot cd to output directory $output_dir";
         
 
-
     my $read_type = ($seqType eq "fq") ? "-q" : "-f";
-
-    if ($left && $right) {
-        $paired_flag = 1;
+        
+    ##############
+    ## Align reads
+    
+    my $bowtie_cmd;
+    
+    if ($aln_method eq 'bowtie') {
+        if ($PE_mode) {
+            my ($left_file, $right_file) = ($sample_href->{left}, $sample_href->{right});
+            ## PE alignment
+            $bowtie_cmd = "set -o pipefail && bowtie $read_type " . $aligner_params{"${aln_method}_${est_method}"} . " -X $max_ins_size -S -p $thread_count $db_index_name -1 $left_file -2 $right_file | samtools view -@ $thread_count -F 4 -S -b | samtools sort -@ $thread_count -n -o $bam_file ";
+            
+        }
+        else {
+            my $single_file = $sample_href->{single};
+            # SE alignment
+            $bowtie_cmd = "set -o pipefail && bowtie $read_type " . $aligner_params{"${aln_method}_${est_method}"} . " -S -p $thread_count $db_index_name $single_file | samtools view -@ $thread_count -F 4 -S -b | samtools sort -@ $thread_count -n -o $bam_file ";
+        }
+    }
+    elsif ($aln_method eq 'bowtie2') {
+        
+        if ($PE_mode) {
+            ## PE alignment
+            my ($left_file, $right_file) = ($sample_href->{left}, $sample_href->{right});
+            $bowtie_cmd = "set -o pipefail && bowtie2 " . $aligner_params{"${aln_method}_${est_method}"} . " $read_type -X $max_ins_size -x $db_index_name -1 $left_file -2 $right_file -p $thread_count | samtools view -@ $thread_count -F 4 -S -b | samtools sort -@ $thread_count -n -o $bam_file ";
+        }
+        else {
+            # SE alignment
+            my $single_file = $sample_href->{single};
+            $bowtie_cmd = "set -o pipefail && bowtie2 " . $aligner_params{"${aln_method}_${est_method}"} . " $read_type -x $db_index_name -U $single_file -p $thread_count | samtools view -@ $thread_count -F 4 -S -b | samtools sort -@ $thread_count -n -o $bam_file ";
+        }
     }
     
-    if (! $PROCESSING_EXISTING_BAM_FLAG) {
-        ## run bowtie
-        
-        ##############
-        ## Align reads
-                
-        my $bowtie_cmd;
-            
-        if ($aln_method eq 'bowtie') {
-            if ($left && $right) {
-                ## PE alignment
-                $bowtie_cmd = "set -o pipefail && bowtie $read_type " . $aligner_params{"${aln_method}_${est_method}"} . " -X $max_ins_size -S -p $thread_count $db_index_name -1 $left -2 $right | samtools view -F 4 -S -b -o $bam_file -";
-                
-            }
-            else {
-                # SE alignment
-                $bowtie_cmd = "set -o pipefail && bowtie $read_type " . $aligner_params{"${aln_method}_${est_method}"} . " -S -p $thread_count $db_index_name $single | samtools view -F 4 -S -b -o $bam_file -";
-            }
-        }
-        elsif ($aln_method eq 'bowtie2') {
-            
-            if ($left && $right) {
-                ## PE alignment
-                $bowtie_cmd = "set -o pipefail && bowtie2 " . $aligner_params{"${aln_method}_${est_method}"} . " $read_type -X $max_ins_size -x $db_index_name -1 $left -2 $right -p $thread_count | samtools view -F 4 -S -b -o $bam_file -";
-            }
-            else {
-                # SE alignment
-                $bowtie_cmd = "set -o pipefail && bowtie2 " . $aligner_params{"${aln_method}_${est_method}"} . " $read_type -x $db_index_name -U $single -p $thread_count | samtools view -F 4 -S -b -o $bam_file -";
-            }
-        }
-        
-        &process_cmd($bowtie_cmd) unless (-s $bam_file && -e $bam_file_ok);
-        
-        &process_cmd("touch $bam_file_ok") unless (-e $bam_file_ok);
-    }
+    &process_cmd($bowtie_cmd) unless (-s $bam_file && -e $bam_file_ok);
     
-    if ($est_method =~ /express/i) {
-        &run_eXpress($bam_file);
-    }
-    elsif ($est_method eq "RSEM") {
-        &run_RSEM($bam_file, $rsem_prefix, $output_prefix);
+    &process_cmd("touch $bam_file_ok") unless (-e $bam_file_ok);
+    
+        
+    if ($est_method eq "RSEM") {
+        
+        # convert bam file for use with rsem:
+        &process_cmd("convert-sam-for-rsem $bam_file $bam_file.for_rsem");
+        
+        &run_RSEM("$bam_file.for_rsem.bam", $rsem_prefix, $output_prefix);
     }
     elsif ($est_method eq "none") {
         print STDERR "Not running abundance estimation, stopping now after alignment.\n";
@@ -617,7 +645,7 @@ sub sort_bam_file {
     if (! -e "$sorted_bam_file.bam.ok") {
         ## sort the bam file
         
-        my $cmd = "samtools sort $bam_file $sorted_bam_file";
+        my $cmd = "samtools sort $bam_file -o $sorted_bam_file.bam";
         &process_cmd($cmd);
         $cmd = "samtools index $sorted_bam_file.bam";
         &process_cmd($cmd);
@@ -629,48 +657,6 @@ sub sort_bam_file {
 }
 
 
-
-####
-sub run_eXpress {
-    my ($bam_file) = @_;
-    
-    my $SS_opt = "";
-    if ($SS_lib_type) {
-        if ($SS_lib_type eq "F") {
-            $SS_opt = "--f-stranded";
-        }
-        elsif ($SS_lib_type eq "R") {
-            $SS_opt = "--r-stranded";
-        }
-        elsif ($SS_lib_type eq "FR") {
-            $SS_opt = "--fr-stranded";
-        }
-        elsif ($SS_lib_type eq "RF") {
-            $SS_opt = "--rf-stranded";
-        }
-    }
-    
-    ## run eXpress
-    my $fraglength_param = "";
-    if ($single) {
-        $fraglength_param = "--frag-len-mean $fragment_length --frag-len-stddev $fragment_std";
-    }
-    
-    my $express_cmd = "express $SS_opt $fraglength_param $eXpress_add_opts $transcripts";
-    
-    my $cmd = "$express_cmd $bam_file";
-    &process_cmd($cmd);
-    
-    if ($gene_trans_map_file) {
-        
-        my $cmd = "$FindBin::RealBin/support_scripts/eXpress_trans_to_gene_results.pl results.xprs $gene_trans_map_file > results.xprs.genes";
-        &process_cmd($cmd);
-    }
-    
-    return;
-}
-
-
 ####
 sub run_RSEM {
     my ($bam_file, $rsem_prefix, $output_prefix) = @_;
@@ -702,7 +688,7 @@ sub run_RSEM {
         $no_qualities_string = "--no-qualities";
     }
 
-    my $paired_flag_text = ($paired_flag) ? "--paired-end" : "";
+    my $paired_flag_text = ($PE_mode) ? "--paired-end" : "";
 
     my $rsem_bam_flag = ($include_rsem_bam) ? "" : "--no-bam-output";
 
@@ -796,6 +782,7 @@ sub add_zcat_gz {
 
 ####
 sub run_kallisto {
+    my @samples = @_;
     
     my $kallisto_index = "$transcripts.kallisto_idx";
     
@@ -808,24 +795,40 @@ sub run_kallisto {
         &process_cmd($cmd);
     }
 
-    if ($left && $right) {
 
-        my $cmd = "kallisto quant -i $kallisto_index $kallisto_add_opts -o $output_dir $left $right";
-        &process_cmd($cmd);
+    if ($SS_lib_type) {
+        # add strand-specific options for kallisto
+        my $kallisto_ss_opt = ($SS_lib_type =~ /^R/) ? "--rf-stranded" : "--fr-stranded";
+        if ($kallisto_add_opts !~ /$kallisto_ss_opt/) {
+            $kallisto_add_opts .= " $kallisto_add_opts";
+        }
     }
-    elsif ($single) {
-        my $cmd = "kallisto quant --single -l $fragment_length -s $fragment_std -i $kallisto_index -o $output_dir $kallisto_add_opts $single";
-        &process_cmd($cmd);
-    }
-    
-    
-    if ( ($left || $single) && $gene_trans_map_file) {
         
-        my $cmd = "$FindBin::RealBin/support_scripts/kallisto_trans_to_gene_results.pl $output_dir/abundance.tsv $gene_trans_map_file > $output_dir/abundance.tsv.genes";
-        &process_cmd($cmd);
+    foreach my $sample_href (@samples) {
+     
+        my ($output_dir, $left_file, $right_file, $single_file) = ($sample_href->{output_dir},
+                                                                   $sample_href->{left},
+                                                                   $sample_href->{right},
+                                                                   $sample_href->{single});
+   
+        if ($left_file && $right_file) {
+            
+            my $cmd = "kallisto quant -i $kallisto_index $kallisto_add_opts -o $output_dir $left_file $right_file";
+            &process_cmd($cmd);
+        }
+        elsif ($single_file) {
+            my $cmd = "kallisto quant -l $fragment_length -s $fragment_std -i $kallisto_index -o $output_dir $kallisto_add_opts --single $single_file";
+            &process_cmd($cmd);
+        }
+        
+        
+        if ($gene_trans_map_file) {
+            
+            my $cmd = "$FindBin::RealBin/support_scripts/kallisto_trans_to_gene_results.pl $output_dir/abundance.tsv $gene_trans_map_file > $output_dir/abundance.tsv.genes";
+            &process_cmd($cmd);
+        }
     }
 
-
     return;
 }
 
@@ -833,8 +836,9 @@ sub run_kallisto {
 
 ####
 sub run_salmon {
+    my (@samples) = @_;
     
-    my $salmon_index = "$transcripts.salmon_${salmon_idx_type}.idx";
+    my $salmon_index = "$transcripts.salmon.idx";
     
     if ( (! $prep_reference) && (! -e $salmon_index)) {
         confess "Error, no salmon index file: $salmon_index, and --prep_reference not set.  Re-run with --prep_reference";
@@ -842,67 +846,170 @@ sub run_salmon {
     if ($prep_reference && ! -e $salmon_index) {
         
         ## Prep salmon index
-        my $cmd;
-        
-        if ($salmon_idx_type eq 'quasi') {
-            $cmd = "salmon index -t $transcripts -i $salmon_index --type quasi -k $salmon_quasi_kmer_length -p $thread_count";
-        }
-        elsif ($salmon_idx_type eq 'fmd') {
-            $cmd = "salmon index -t $transcripts -i $salmon_index --type fmd -p $thread_count";
-        }
-        else {
-            die "Error, not recognizing idx type: $salmon_idx_type";
-        }
+        my $cmd = "salmon index -t $transcripts --keepDuplicates -i $salmon_index -k $salmon_kmer_length -p $thread_count";
         
         &process_cmd($cmd);
     }
 
-    my $outdir = $output_dir; #"$output_dir.$salmon_idx_type";
+    my $num_failures = 0;
     
-    
-    if ($left && $right) {
-        ## PE mode
-        my $cmd;
-        my $libtype = ($SS_lib_type) ? "IS" . substr($SS_lib_type, 0, 1) : "IU";
+    foreach my $sample_href (@samples) {
+        
+        my ($output_dir, $left_file, $right_file, $single_file) = ($sample_href->{output_dir},
+                                                                   $sample_href->{left},
+                                                                   $sample_href->{right},
+                                                                   $sample_href->{single});
+        
+
+        
+        my $outdir = $output_dir; #"$output_dir.$salmon_idx_type";
+
+        if (-s "$outdir/quant.sf") {
+            print STDERR "-output already exists: $outdir/quant.sf, skipping.\n";
+            next;
+        }
+        
+
+        eval {
+        
+            if ($left_file && $right_file) {
+                ## PE mode
+                my $libtype = ($SS_lib_type) ? "IS" . substr($SS_lib_type, 0, 1) : "IU";
+                
+                my $cmd = "salmon quant -i $salmon_index -l $libtype -1 $left_file -2 $right_file -o $outdir $salmon_add_opts -p $thread_count --validateMappings ";
+                
+                &process_cmd($cmd);
+                
+            }
+            elsif ($single_file) {
+                my $libtype = ($SS_lib_type) ? "S" . substr($SS_lib_type, 0, 1) : "U";
+                my $cmd = "salmon quant -i $salmon_index -l $libtype -r $single_file -o $outdir $salmon_add_opts -p $thread_count --validateMappings ";
+                &process_cmd($cmd);
+                
+            }
             
-        if ($salmon_idx_type eq 'quasi') {
-            $cmd = "salmon quant -i $salmon_index -l $libtype -1 $left -2 $right -o $outdir $salmon_add_opts -p $thread_count";
+            if ($gene_trans_map_file) {
+                
+                my $cmd = "$FindBin::RealBin/support_scripts/salmon_trans_to_gene_results.pl $output_dir/quant.sf $gene_trans_map_file > $output_dir/quant.sf.genes";
+                &process_cmd($cmd);
+            }
+        };
+        if ($@) {
+            $num_failures++;
+            print STDERR "Error detected: $@";
         }
-        elsif ($salmon_idx_type eq 'fmd') {
-            $cmd = "salmon quant -i $salmon_index -l $libtype -1 $left -2 $right -k $salmon_fmd_kmer_length -o $outdir $salmon_add_opts -p $thread_count";
-        }
-        else {
-            die "Error, not recognizing salmon_idx_type: $salmon_idx_type";
-        }
-        
-        &process_cmd($cmd);
-        
     }
-    elsif ($single) {
-        my $libtype = ($SS_lib_type) ? "S" . substr($SS_lib_type, 0, 1) : "U";
-        my $cmd;
-        
-        if ($salmon_idx_type eq 'quasi') {
-            $cmd = "salmon quant -i $salmon_index -l $libtype -r $single -o $outdir $salmon_add_opts -p $thread_count";
-        }
-        elsif ($salmon_idx_type eq 'fmd') {
-            $cmd = "salmon quant -i $salmon_index -l $libtype -r $single -k $salmon_fmd_kmer_length -o $outdir $salmon_add_opts -p $thread_count";
-        }
-        else {
-            die "Error, not recognizing salmon_idx_type: $salmon_idx_type";
-        }
-        
-        &process_cmd($cmd);
 
+
+    if ($num_failures) {
+        die "Error, encountered $num_failures failed salmon jobs. See errors above";
     }
     
-    if ( ($left || $single) && $gene_trans_map_file) {
-        
-        my $cmd = "$FindBin::RealBin/support_scripts/salmon_trans_to_gene_results.pl $output_dir/quant.sf $gene_trans_map_file > $output_dir/quant.sf.genes";
-        &process_cmd($cmd);
-    }
-    
-
     return;
 }
 
+
+
+####
+sub parse_samples_file {
+    my ($samples_file) = @_; 
+
+    my @samples_to_process;
+
+    my %seen;
+    open (my $fh, $samples_file) or die "Error, cannot open file: [$samples_file]";
+    while (<$fh>) {
+        chomp;
+        if (/^\#/) { next; }
+        unless (/\w/) { next; }
+        if (/^\-/) { next; } 
+        s/^\s+|\s+$//g; # trim trailing ws
+        my @x = split(/\s+/);
+
+        my $sample_name = $x[0];
+        my $rep_name = $x[1];
+        if ($seen{$rep_name}) {
+            die "Error, replicate names must be unique.  Found $rep_name listed multiple times";
+        }
+        $seen{$rep_name}++;
+        
+        my $output_dir = $rep_name;
+        
+        my $left_fq = $x[2];
+        my $right_fq = $x[3];
+        
+        if ($left_fq) {
+            unless (-s $left_fq) {
+                die "Error, cannot locate file: $left_fq as specified in samples file: $samples_file";
+            }
+            $left_fq = &create_full_path($left_fq);
+            if ($left_fq =~ /\.gz$/) {
+                $left_fq = &add_zcat_gz($left_fq) if ($aln_method eq "bowtie");
+            }
+        }
+        else {
+            die "Error, cannot parse line $_ of samples file: $samples_file . See usage info for samples file formatting requirements.";
+            
+        }
+        if ($right_fq) {
+            unless (-s $right_fq) {
+                die "Error, cannot locate file $right_fq as specified in samples file: $samples_file";
+            }
+            $right_fq = &create_full_path($right_fq);
+            if ($right_fq =~ /\.gz$/) {
+                $right_fq = &add_zcat_gz($right_fq) if ($aln_method eq "bowtie");
+            } 
+        }
+
+        if ($left_fq && $right_fq) {
+
+            push (@samples_to_process, { left => $left_fq,
+                                         right => $right_fq,
+                                         output_dir => $output_dir,
+                  } );
+        }
+        else {
+            push (@samples_to_process, { single => $left_fq,
+                                         output_dir => $output_dir,
+                  } );
+        }
+        
+    }
+   
+    
+    return (@samples_to_process);
+}
+
+
+####
+sub create_sample_definition {
+    my ($output_dir, $left, $right, $single) = @_;
+
+    $left = &create_full_path($left) if $left;
+    $right = &create_full_path($right) if $right;
+    $single = &create_full_path($single) if $single;
+
+    if ($left && $left =~ /\.gz$/) {
+        $left = &add_zcat_gz($left) if ($aln_method eq "bowtie");
+    }
+    if ($right && $right =~ /\.gz$/) {
+        $right = &add_zcat_gz($right) if ($aln_method eq "bowtie");
+    }
+    if ($single && $single =~ /\.gz$/) {
+        $single = &add_zcat_gz($single) if ($aln_method eq "bowtie");
+    }
+    
+    
+    if ($left && $right) {
+        return ( { left => $left,
+                   right => $right,
+                   output_dir => $output_dir,
+                 } );
+    }
+    else {
+        return( { single => $single,
+                  output_dir => $output_dir,
+                } );
+    }
+
+}
