@@ -43,10 +43,14 @@ if ($help_flag) {
     die $usage;
 }
 
-unless ($reads_file && -s $reads_file) {
+unless ($reads_file && -e $reads_file) {
     die $usage;
 }
 
+unless (-s $reads_file) {
+    die "Error, reads file listing: $reads_file is empty.  This tends to happen when there were too few reads to assemble. ";
+}
+
 
 my $trin_args = "";
 while (@TRIN_ARGS) {
@@ -56,7 +60,7 @@ while (@TRIN_ARGS) {
         my $val = shift @TRIN_ARGS;
         # retain quotes around multiparams
         
-        $trin_args .= "$arg \"$val\"";
+        $trin_args .= "$arg \"$val\" ";
     }
     else {
         $trin_args .= "$arg ";
